from __future__ import absolute_import
from importlib import import_module

from catchpoint.config.config_provider import ConfigProvider
from catchpoint.config import config_names
import wrapt

_catchpoint_instance = None


def _wrapper(wrapped, _, args, kwargs):
    wrapped = _catchpoint_instance(wrapped)
    return wrapped(*args, **kwargs)


def patch(catchpoint_instance):
    if (not ConfigProvider.get(config_names.CATCHPOINT_TRACE_INTEGRATIONS_CHALICE_DISABLE)) and catchpoint_instance:
        global _catchpoint_instance
        _catchpoint_instance = catchpoint_instance
        try:
            import_module("chalice")
            wrapt.wrap_function_wrapper("chalice", "Chalice.__call__", _wrapper)
        except:
            pass



from catchpoint.listeners.catchpoint_span_listener import CatchpointSpanListener
from catchpoint.listeners.error_injector_span_listener import ErrorInjectorSpanListener
from catchpoint.listeners.latency_injector_span_listener import LatencyInjectorSpanListener
from catchpoint.listeners.filtering_span_listener import FilteringSpanListener
from catchpoint.listeners.tag_injector_span_listener import TagInjectorSpanListener
from catchpoint.listeners.security_aware_span_listener import SecurityAwareSpanListener



import wrapt

from catchpoint.integrations.sqlite3 import SQLiteIntegration
from catchpoint.config import config_names
from catchpoint.config.config_provider import ConfigProvider

sqlite_integration = SQLiteIntegration()


class SqliteCursorWrapper(wrapt.ObjectProxy):

    def __init__(self, cursor, connection_wrapper):
        super(SqliteCursorWrapper, self).__init__(cursor)
        self._self_connection = connection_wrapper

    def execute(self, *args, **kwargs):
        return sqlite_integration.run_and_trace(
            self.__wrapped__.execute,
            self._self_connection,
            args,
            kwargs,
        )

    def __enter__(self):
        # raise appropriate error if api not supported (should reach the user)
        self.__wrapped__.__enter__
        return self


class SqliteConnectionWrapper(wrapt.ObjectProxy):
    db_name = None
    host = None

    def __init__(self, connection, db_name, host='localhost'):
        super(SqliteConnectionWrapper, self).__init__(connection)
        self.db_name = db_name
        self.host = host

    def cursor(self):
        cursor = self.__wrapped__.cursor()
        return SqliteCursorWrapper(cursor, self)

    def execute(self, *args, **kwargs):
        return self.cursor().execute(*args, **kwargs)


def _wrapper(wrapped, instance, args, kwargs):
    db_name = args[0] if args and len(args) > 0 else None
    connection = wrapped(*args, **kwargs)
    return SqliteConnectionWrapper(connection, db_name)


def patch():
    if not ConfigProvider.get(config_names.CATCHPOINT_TRACE_INTEGRATIONS_RDB_DISABLE):
        wrapt.wrap_function_wrapper(
            'sqlite3',
            'connect',
            _wrapper)
        wrapt.wrap_function_wrapper(
            'sqlite3.dbapi2',
            'connect',
            _wrapper)



